@@ -50,7 +50,6 @@ import apt_pkg
 import os
 import re
 import logging
-import time
 import threading
 
 from gettext import gettext as _
@@ -198,14 +197,9 @@ class CellAreaPackage(Gtk.CellAreaBox):
         if not self.column:
             return 0
         else:
-            val = GObject.Value()
-            val.init(int)
-            widget.style_get_property("horizontal-separator", val)
-            h_sep = val.get_int()
-            widget.style_get_property("grid-line-width", val)
-            line_width = val.get_int()
-            cell_start = self.column.get_x_offset() - h_sep - line_width
+            cell_start = self.column.get_x_offset()
             if not self.indent_toplevel:  # i.e. if no headers
+                val = GObject.Value()
                 widget.style_get_property("expander-size", val)
                 spacing = self.get_property("spacing")
                 # Hardcode 4 because GTK+ hardcodes 4 internally
@@ -308,7 +302,9 @@ class UpdatesAvailable(InternalDialog):
 
         pkg_toggle_renderer = Gtk.CellRendererToggle()
         pkg_toggle_renderer.set_property("ypad", 2)
-        pkg_toggle_renderer.connect("toggled", self.on_update_toggled)
+        pkg_toggle_renderer.connect(
+            "toggled", self.on_update_toggled, self.treeview_update
+        )
         pkg_column.pack_start(pkg_toggle_renderer, False)
         pkg_column.add_attribute(
             pkg_toggle_renderer, "active", LIST_TOGGLE_ACTIVE
@@ -397,6 +393,9 @@ class UpdatesAvailable(InternalDialog):
             apt_news = self._get_apt_news(
                 "/var/lib/ubuntu-advantage/messages/apt-news"
             )
+        except Exception as e:
+            print("Error running current_news end-point: ", e)
+            apt_news = ""
         if apt_news:
             self.news.get_buffer().set_text(apt_news)
             self.expander_news.set_visible(True)
@@ -411,6 +410,10 @@ class UpdatesAvailable(InternalDialog):
         self.alert_watcher.check_alert_state()
         self._restore_state()
 
+        if self.window_main.options.install_all:
+            self.window_main.options.install_all = False
+            self.install_all_updates(None, None, None)
+
     def is_auto_update(self):
         update_days = apt_pkg.config.find_i(
             "APT::Periodic::Update-Package-Lists"
@@ -604,6 +607,13 @@ class UpdatesAvailable(InternalDialog):
         model = widget.get_model()
         iter = model.get_iter(path)
 
+        # Screen reader: Speak state of toggle box
+        ac = widget.get_accessible()
+        if model.get_value(iter, LIST_TOGGLE_ACTIVE):
+            ac.emit("notification", _("Selected"), 0)
+        else:
+            ac.emit("notification", _("Unselected"), 0)
+
         # set descr
         data = model.get_value(iter, LIST_UPDATE_DATA)
         item = data.item
@@ -657,12 +667,29 @@ class UpdatesAvailable(InternalDialog):
                     "changelog information."
                 )
             )
-        # else, get it from the entwork
+            return
+        # else, get it from the network
         elif self.expander_details.get_expanded():
-            lock = threading.Lock()
-            lock.acquire()
+            button = Gtk.Button(stock="gtk-cancel")
+            callback_args = (path, widget, name, changes_buffer, long_desc)
+            id = button.connect("clicked", lambda _, args: hide_button(args),
+                                callback_args)
+
+            def hide_button(args):
+                nonlocal id
+                if id is not None:
+                    button.disconnect(id)
+                    id = None
+                    button.set_visible(0)
+                    self.display_changes(args)
+
+            def on_thread_done(args):
+                GLib.idle_add(hide_button, args)
+
             changelog_thread = threading.Thread(
-                target=self.cache.get_news_and_changelog, args=(name, lock)
+                target=self.cache.get_news_and_changelog,
+                args=(name,),
+                kwargs={'cb': on_thread_done, 'cb_args': callback_args}
             )
             changelog_thread.start()
             changes_buffer.set_text(
@@ -670,21 +697,14 @@ class UpdatesAvailable(InternalDialog):
             )
             iter = changes_buffer.get_iter_at_line(1)
             anchor = changes_buffer.create_child_anchor(iter)
-            button = Gtk.Button(stock="gtk-cancel")
             self.textview_changes.add_child_at_anchor(button, anchor)
             button.show()
-            id = button.connect(
-                "clicked", lambda w, lock: lock.release(), lock
-            )
-            # wait for the dl-thread
-            while lock.locked():
-                time.sleep(0.01)
-                while Gtk.events_pending():
-                    Gtk.main_iteration()
-            # download finished (or canceld, or time-out)
-            button.hide()
-            if button.handler_is_connected(id):
-                button.disconnect(id)
+            return
+        args_normal = (path, widget, name, changes_buffer, long_desc)
+        self.display_changes(args_normal)
+
+    def display_changes(self, args):
+        (path, widget, name, changes_buffer, long_desc) = args
         # check if we still are in the right pkg (the download may have taken
         # some time and the user may have clicked on a new pkg)
         now_path = widget.get_cursor()[0]
@@ -780,8 +800,6 @@ class UpdatesAvailable(InternalDialog):
             )
         else:
             self.window_main.get_window().set_cursor(None)
-        while Gtk.events_pending():
-            Gtk.main_iteration()
 
     def _mark_selected_updates(self):
         def foreach_cb(model, path, iter, data):
@@ -818,6 +836,7 @@ class UpdatesAvailable(InternalDialog):
         self.hbox_restart.set_visible(requires_restart)
 
     def _refresh_updates_count(self):
+        self.button_install.show()
         self.button_install.set_sensitive(
             self.cache.install_count + self.cache.del_count
         )
@@ -853,7 +872,7 @@ class UpdatesAvailable(InternalDialog):
                       and not self.list.ubuntu_pro_groups):
                     download_str = _("You need to enable Ubuntu Pro to install"
                                      " these updates.")
-                    self.button_install.destroy()
+                    self.button_install.hide()
                 else:
                     download_str = _("There are no updates to install.")
                     self.button_install.set_sensitive(False)
@@ -1064,7 +1083,7 @@ class UpdatesAvailable(InternalDialog):
             self.hbox_on_3g.hide()
             self.hbox_roaming.hide()
 
-    def on_update_toggled(self, renderer, path):
+    def on_update_toggled(self, renderer, path, treeview):
         """a toggle button in the listview was toggled"""
         iter = self.store.get_iter(path)
         data = self.store.get_value(iter, LIST_UPDATE_DATA)
@@ -1079,12 +1098,19 @@ class UpdatesAvailable(InternalDialog):
         else:
             self.toggle_from_items([data.item])
 
+        # Screen reader: Speak state of toggle box
+        ac = treeview.get_accessible()
+        if self.store.get_value(iter, LIST_TOGGLE_ACTIVE):
+            ac.emit("notification", _("Selected"), 0)
+        else:
+            ac.emit("notification", _("Unselected"), 0)
+
     def on_treeview_update_row_activated(self, treeview, path, column, *args):
         """
         If an update row was activated (by pressing space), toggle the
         install check box
         """
-        self.on_update_toggled(None, path)
+        self.on_update_toggled(None, path, treeview)
 
     def toggle_from_items(self, items):
         self.setBusy(True)
@@ -1252,7 +1278,5 @@ class UpdatesAvailable(InternalDialog):
         self.update_close_button()
         self.update_count()
         self.setBusy(False)
-        while Gtk.events_pending():
-            Gtk.main_iteration()
         self.updates_changed()
         return False
